@@ -118,6 +118,7 @@ struct ploop_disk_images_data {
 struct ploop_resize_param {
 	unsigned long long size;
 	int offline_resize;
+	pid_t mntns_pid;
 	char dummy[32];
 };
 
@@ -165,6 +166,7 @@ struct ploop_discard_param {
 	__u64 to_free;
 	int automount;
 	const int *stop;
+	int defrag;
 	char dummy[32];
 };
 
@@ -231,6 +233,7 @@ extern "C" {
 int ploop_set_component_name(struct ploop_disk_images_data *di,
 		const char *component_name);
 int ploop_get_top_delta_fname(struct ploop_disk_images_data *di, char *out, int len);
+int ploop_get_base_delta_fname(struct ploop_disk_images_data *di, char *out, int len);
 int ploop_find_dev(const char *component_name, const char *delta, char *buf, int size);
 int ploop_get_dev_by_mnt(const char *path, char *buf, int size);
 int ploop_get_mnt_by_dev(const char *dev, char *buf, int size);
@@ -279,8 +282,6 @@ void ploop_set_verbose_level(int level);
 
 /* Cancelation API */
 void ploop_cancel_operation(void);
-int ploop_complete_running_operation(const char *device);
-
 /* pcopy routines */
 int ploop_copy_send(struct ploop_copy_send_param *arg);
 int ploop_copy_receive(struct ploop_copy_receive_param *arg);
@@ -301,6 +302,7 @@ int ploop_read_disk_descr(struct ploop_disk_images_data **di, const char *file);
 PLOOP_DEPRECATED char *ploop_get_base_delta_uuid(struct ploop_disk_images_data *di);
 PLOOP_DEPRECATED int ploop_send(const char *device, int ofd, const char *flush_cmd, int is_pipe);
 PLOOP_DEPRECATED int ploop_receive(const char *dst);
+PLOOP_DEPRECATED int ploop_complete_running_operation(const char *device);
 
 #ifdef __cplusplus
 }
