@@ -41,6 +41,8 @@
 
 #define EXT4_IOC_OPEN_BALLOON		_IO('f', 42)
 
+#define BIN_E4DEFRAG	"/usr/sbin/e4defrag2"
+
 
 char *mntn2str(int mntn_type)
 {
@@ -833,7 +835,7 @@ static void cancel_discard(void *data)
 static int ploop_trim(const char *mount_point, __u64 minlen_b, __u64 cluster)
 {
 	struct fstrim_range range = {};
-	int fd, ret = -1;
+	int fd, ret = -1, last = 0;
 
 	struct sigaction sa = {
 		.sa_handler     = stop_trim_handler,
@@ -873,9 +875,14 @@ static int ploop_trim(const char *mount_point, __u64 minlen_b, __u64 cluster)
 			break;
 		}
 
+		if (last)
+			break;
+
 		/* last iteration should go with range.minlen == minlen_b */
-		if (range.minlen != minlen_b && range.minlen / 2 < minlen_b)
+		if (range.minlen != minlen_b && range.minlen / 2 < minlen_b) {
 			range.minlen = minlen_b * 2;
+			last = 1;
+		}
 	}
 
 	close(fd);
@@ -909,13 +916,42 @@ static int blk_discard(int fd, __u32 cluster, __u64 start, __u64 len)
 	return 0;
 }
 
+static int wait_pid(pid_t pid, const char *mes)
+{
+	int err, status;
+
+	while ((err = waitpid(pid, &status, 0)))
+		 if (errno != EINTR)
+			break;
+	if (err == -1) {
+		if (errno != ECHILD)
+			ploop_err(errno, "wait() failed");
+	} else if (WIFEXITED(status)) {
+		err = WEXITSTATUS(status);
+		if (err) {
+			ploop_err(0, "The %s process failed with code %d",
+					mes, err);
+			err = -1;
+		}
+	} else if (WIFSIGNALED(status)) {
+		ploop_err(0, "The %s process killed by signal %d",
+				mes, WTERMSIG(status));
+		err = -1;
+	} else {
+		ploop_err(0, "The %s process died abnormally", mes);
+		err = -1;
+	}
+
+	return err;
+}
+
 static int __ploop_discard(struct ploop_disk_images_data *di, int fd,
 			const char *device, const char *mount_point,
 			__u64 minlen_b, __u32 cluster, __u32 to_free,
 			__u64 blk_discard_range[2], const int *stop)
 {
 	pid_t tpid;
-	int err = 0, ret, status;
+	int ret;
 	__u32 size = 0;
 	struct ploop_cleanup_hook *h;
 
@@ -981,7 +1017,7 @@ static int __ploop_discard(struct ploop_disk_images_data *di, int fd,
 			continue;
 		} else
 			size += ret;
-		/* serialize ploop operations vs ploop_complete_running_operation()
+		/* serialize ploop operations vs complete_running_operation()
 		 * NB: PLOOP_IOC_BALLOON may change mntn from PLOOP_MNTN_DISCARD:
 		 * to PLOOP_MNTN_FBLOADED
 		 */
@@ -1021,8 +1057,6 @@ static int __ploop_discard(struct ploop_disk_images_data *di, int fd,
 	}
 
 	if (ret) {
-		err = -1;
-
 		ret = ioctl(fd, PLOOP_IOC_DISCARD_FINI);
 		if (ret < 0) {
 			if (errno == EBUSY)
@@ -1039,28 +1073,7 @@ static int __ploop_discard(struct ploop_disk_images_data *di, int fd,
 
 	unregister_cleanup_hook(h);
 
-	while ((ret = waitpid(tpid, &status, 0)))
-		 if (errno != EINTR)
-			break;
-	if (ret == -1) {
-		if (errno != ECHILD)
-			ploop_err(errno, "wait() failed");
-		err = -1;
-	} else if (WIFEXITED(status)) {
-		ret = WEXITSTATUS(status);
-		if (ret) {
-			ploop_err(0, "The trim process failed with code %d", ret);
-			err = -1;
-		}
-	} else if (WIFSIGNALED(status)) {
-		ploop_err(0, "The trim process killed by signal %d", WTERMSIG(status));
-		err = -1;
-	} else {
-		ploop_err(0, "The trim process died abnormally");
-		err = -1;
-	}
-
-	return err;
+	return wait_pid(tpid, "trim");
 }
 
 static int do_ploop_discard(struct ploop_disk_images_data *di,
@@ -1155,6 +1168,129 @@ int ploop_discard_by_dev(const char *device, const char *mount_point,
 	return do_ploop_discard(NULL, device, mount_point, minlen_b, to_free, stop);
 }
 
+static void defrag_pidfile(const char *dev, char *out, int size)
+{
+	char *p = strrchr(dev, '/');
+
+	snprintf(out, size, PLOOP_LOCK_DIR "/%s.defrag.pid",
+			p ? ++p : dev);
+}
+
+static void defrag_complete(const char *dev)
+{
+	char buf[PATH_MAX];
+	char cmdline[64];
+	pid_t pid;
+	FILE *fp;
+	char cmd[64];
+	BUILD_BUG_ON(sizeof(cmd) <= sizeof(BIN_E4DEFRAG));
+
+	defrag_pidfile(dev, buf, sizeof(buf));
+
+	fp = fopen(buf, "r");
+	if (fp == NULL) {
+		if (errno != ENOENT)
+			ploop_err(errno, "Can't open %s", buf);
+		return;
+	}
+
+	if (fscanf(fp, "%d\n", &pid) != 1) {
+		ploop_err(0, "Can't read PID from %s", buf);
+		fclose(fp);
+		return;
+	}
+	fclose(fp);
+
+	snprintf(cmdline, sizeof(cmdline), "/proc/%d/cmdline", pid);
+	if (read_line(cmdline, cmd, sizeof(cmd))) {
+		// error is printed by read_line()
+		goto stale;
+	}
+
+	// we rely on the fact that cmdline has \0 after argv[0]
+	if (strcmp(BIN_E4DEFRAG, cmd) != 0) {
+		// some other process that happen to reuse our pid
+		goto stale;
+	}
+
+	ploop_log(0, "Cancelling defrag dev=%s pid=%d", dev, pid);
+	kill(pid, SIGTERM);
+	return;
+
+stale:
+	if (access(buf, F_OK))
+		ploop_log(0, "Warning: stale defrag pidfile %s", buf);
+
+	return;
+}
+
+static int create_pidfile(const char *fname, pid_t pid)
+{
+	FILE *fp;
+
+	fp = fopen(fname, "w");
+	if (fp == NULL) {
+		ploop_err(errno, "Cant't create %s", fname);
+		return -1;
+	}
+
+	fprintf(fp, "%d\n", pid);
+
+	fclose(fp);
+
+	return 0;
+}
+
+static int ploop_defrag(struct ploop_disk_images_data *di,
+		const char *dev, const char *mnt)
+{
+	int ret;
+	int blocksize;
+	pid_t pid;
+	char pidfile[PATH_MAX];
+	char block_size[16];
+	char part[64];
+	char *arg[] = {BIN_E4DEFRAG, "-c", block_size, part, (char*)mnt, NULL};
+
+	if (access(arg[0], F_OK))
+		return 0;
+
+	if (ploop_get_attr(dev, "block_size", &blocksize)) {
+		ploop_err(0, "Can't find block size");
+		return -1;
+	}
+
+	snprintf(block_size, sizeof(block_size), "%d", blocksize << 9);
+
+	if (ploop_get_partition_by_mnt(mnt, part, sizeof(part))) {
+		ploop_log(-1, "Can't get partition by_mnt %s", mnt);
+		return -1;
+	}
+
+	ploop_log(0, "Start defrag dev=%s mnt=%s blocksize=%d",
+			part, mnt, blocksize);
+
+	pid = fork();
+	if (pid < 0) {
+		ploop_err(errno, "Can't fork");
+		return -1;
+	} else if (pid == 0) {
+		execv(arg[0], arg);
+
+		ploop_err(errno, "Can't exec %s", arg[0]);
+		_exit(1);
+	}
+
+	defrag_pidfile(dev, pidfile, sizeof(pidfile));
+	create_pidfile(pidfile, pid);
+
+	ret = wait_pid(pid, arg[0]);
+
+	unlink(pidfile);
+
+	return ret;
+}
+
 int ploop_discard(struct ploop_disk_images_data *di,
 		struct ploop_discard_param *param)
 {
@@ -1198,6 +1334,26 @@ int ploop_discard(struct ploop_disk_images_data *di,
 	}
 	ploop_unlock_dd(di);
 
+	if (param->defrag) {
+		struct ploop_discard_stat pds, pds_after;
+
+		ret = ploop_discard_get_stat_by_dev(dev, mnt, &pds);
+
+		if (ploop_defrag(di, dev, mnt))
+			ploop_log(0, BIN_E4DEFRAG" exited with error");
+
+		ret += ploop_discard_get_stat_by_dev(dev, mnt, &pds_after);
+		if (ret)
+			ploop_log(0, "Unable to get ploop stats");
+
+		/*
+		 * Take into account possible change of ploop image size caused
+		 * by defrag operation defrag operation.
+		 */
+		if (param->to_free && !ret)
+			param->to_free += pds_after.image_size - pds.image_size;
+	}
+
 	ret = do_ploop_discard(di, dev, mnt, param->minlen_b,
 			param->to_free, param->stop);
 
@@ -1210,10 +1366,80 @@ int ploop_discard(struct ploop_disk_images_data *di,
 }
 
 int ploop_complete_running_operation(const char *device)
+{
+	return 0;
+}
+
+static int do_mntn_merge(struct ploop_disk_images_data *di, const char *device,
+		int fd)
+{
+	int ret;
+	char x[PATH_MAX];
+	char conf[PATH_MAX];
+	char *top_delta = NULL;
+
+	if (di == NULL) {
+		ploop_err(0, "Unable to complete on-going merge operation:"
+			" DiskDescriptor.xml is not provided");
+		return SYSEXIT_PARAM;
+	}
+
+	get_disk_descriptor_fname(di, conf, sizeof(conf));
+	ploop_log(0, "Process PLOOP_MNTN_MERGE state on '%s'", conf);
+
+	ret = ploop_find_top_delta_name_and_format(device, x, sizeof(x), NULL, 0);
+	if (ret)
+		return ret;
+
+	/* make validation before real merge */
+	ret = ploop_di_merge_image(di, di->top_guid);
+	if (ret)
+		return ret;
+
+	top_delta = find_image_by_guid(di, di->top_guid);
+
+	ploop_log(0, "Repair %s: merge top delta %s", conf, top_delta);
+	if (ploop_fname_cmp(x, top_delta)) {
+		ploop_err(0, "Config %s inconsistent with device state: "
+				"top delta file differs "
+				"(device='%s' config='%s')\n",
+				conf, x, top_delta);
+		ret = SYSEXIT_DISKDESCR;
+		goto err;
+	}
+
+	snprintf(x, sizeof(x), "%s.tmp", conf);
+	ret = ploop_store_diskdescriptor(x, di);
+	if (ret)
+		goto err;
+
+	ret = ioctl_device(fd, PLOOP_IOC_MERGE, 0);
+	if (ret)
+		goto err;
+
+	if (rename(x, conf)) {
+		ploop_err(errno, "Can't rename %s %s", x, conf);
+		ret = SYSEXIT_RENAME;
+		goto err;
+	}
+
+	if (unlink(top_delta))
+		 ploop_err(errno, "Can't unlink %s", top_delta);
+
+err:
+	free(top_delta);
+
+	return ret;
+}
+
+int complete_running_operation(struct ploop_disk_images_data *di,
+		const char *device)
 {
 	struct ploop_balloon_ctl b_ctl;
 	int fd, ret;
 
+	defrag_complete(device);
+
 	fd = open_device(device);
 	if (fd == -1)
 		return SYSEXIT_OPEN;
@@ -1234,7 +1460,7 @@ int ploop_complete_running_operation(const char *device)
 
 	switch (b_ctl.mntn_type) {
 	case PLOOP_MNTN_MERGE:
-		ret = ioctl_device(fd, PLOOP_IOC_MERGE, 0);
+		ret = do_mntn_merge(di, device, fd);
 		break;
 	case PLOOP_MNTN_GROW:
 		ret = ioctl_device(fd, PLOOP_IOC_GROW, 0);
