@@ -341,9 +341,6 @@ static int check_size(unsigned long long sectors, __u32 blocksize, int version)
 
 int check_blockdev_size(unsigned long long sectors, __u32 blocksize, int version)
 {
-	if (check_size(sectors, blocksize, version))
-		return -1;
-
 	if (sectors % blocksize) {
 		ploop_err(0, "An incorrect block device size is specified: %llu sectors."
 				" The block device size must be aligned to the cluster block size %d",
@@ -429,9 +426,6 @@ int create_snapshot_delta(const char *path, __u32 blocksize, off_t bdsize,
 	if (version == PLOOP_FMT_UNDEFINED)
 		version = default_fmt_version();
 
-	if (do_check_size(bdsize, blocksize, version, 0))
-		return -1;
-
 	return do_create_delta(path, blocksize, bdsize, version);
 }
 
@@ -757,7 +751,8 @@ static int init_dd(struct ploop_disk_images_data **di,
 		return SYSEXIT_PARAM;
 	}
 
-	if (check_size(param->size, blocksize, fmt_version))
+	if (param->fstype != NULL &&
+			check_size(param->size, blocksize, fmt_version))
 		return SYSEXIT_PARAM;
 
 	if (!is_valid_blocksize(blocksize)) {
@@ -1075,7 +1070,7 @@ void unmangle_to_buffer(const char *s, char *buf, size_t len)
  *  1 mount point not found (fs not mounted)
  * -1 some system error
  */
-static int get_mount_dir(const char *device, char *out, int size)
+static int get_mntns_mount_dir(const char *device, int pid, char *out, int size)
 {
 	FILE *fp;
 	int ret = 1;
@@ -1087,14 +1082,18 @@ static int get_mount_dir(const char *device, char *out, int size)
 
 	if (get_dev_by_name(device, &dev))
 		return -1;
-	minor = gnu_dev_minor(dev);
 
-	fp = fopen("/proc/self/mountinfo", "r");
+	if (pid > 0)
+		snprintf(buf, sizeof(buf), "/proc/%d/mountinfo", pid);
+	else
+		snprintf(buf, sizeof(buf), "/proc/self/mountinfo");
+	fp = fopen(buf, "r");
 	if (fp == NULL) {
-		ploop_err(errno, "Can't open /proc/self/mountinfo");
+		ploop_err(errno, "Can't open %s", buf);
 		return -1;
 	}
 
+	minor = minor(dev);
 	while (fgets(buf, sizeof(buf), fp)) {
 		n = sscanf(buf, "%u %u %u:%u %*s %4096s", &u, &u, &_major, &_minor, target);
 		if (n != 5)
@@ -1103,7 +1102,8 @@ static int get_mount_dir(const char *device, char *out, int size)
 		if (_major == PLOOP_DEV_MAJOR &&
 				(_minor == minor || _minor == minor + 1))
 		{
-			unmangle_to_buffer(target, out, size);
+			if (out != NULL)
+				unmangle_to_buffer(target, out, size);
 			ret = 0;
 			break;
 		}
@@ -1112,6 +1112,11 @@ static int get_mount_dir(const char *device, char *out, int size)
 	return ret;
 }
 
+static int get_mount_dir(const char *device, char *out, int size)
+{
+	return get_mntns_mount_dir(device, 0, out, size);
+}
+
 int ploop_get_mnt_by_dev(const char *dev, char *buf, int size)
 {
 	return get_mount_dir(dev, buf, size);
@@ -1197,26 +1202,41 @@ static const char *get_top_delta_guid(struct ploop_disk_images_data *di)
 	return di->top_guid;
 }
 
-int ploop_get_top_delta_fname(struct ploop_disk_images_data *di, char *out, int len)
+static int get_delta_fname(struct ploop_disk_images_data *di, const char *guid,
+	char *out, int len)
 {
-	int ret = 0;
 	const char *fname;
 
-	if (ploop_lock_dd(di))
-		return SYSEXIT_LOCK;
-
-	fname = find_image_by_guid(di, get_top_delta_guid(di));
+	fname = find_image_by_guid(di, guid);
 	if (fname == NULL){
-		ploop_err(0, "Can't find image by uuid %s", di->top_guid);
-		ret = SYSEXIT_PARAM;
-		goto out;
+		ploop_err(0, "Can't find image by uuid %s", guid);
+		return SYSEXIT_PARAM;
 	}
 	if (snprintf(out, len, "%s", fname) > len -1) {
 		ploop_err(0, "Not enough space to store data");
-		ret = SYSEXIT_PARAM;
+		return SYSEXIT_PARAM;
 	}
+	return 0;
+}
 
-out:
+int ploop_get_top_delta_fname(struct ploop_disk_images_data *di, char *out, int len)
+{
+	int ret;
+	if (ploop_lock_dd(di))
+		return SYSEXIT_LOCK;
+
+	ret = get_delta_fname(di, get_top_delta_guid(di), out, len);
+	ploop_unlock_dd(di);
+	return ret;
+}
+
+int ploop_get_base_delta_fname(struct ploop_disk_images_data *di, char *out, int len)
+{
+	int ret;
+	if (ploop_lock_dd(di))
+		return SYSEXIT_LOCK;
+
+	ret = get_delta_fname(di, get_base_delta_uuid(di), out, len);
 	ploop_unlock_dd(di);
 	return ret;
 }
@@ -1250,6 +1270,9 @@ int ploop_is_mounted(struct ploop_disk_images_data *di)
 
 int ploop_get_devs(struct ploop_disk_images_data *di, char ***out)
 {
+	if (di->nimages == 0 && ploop_read_dd(di))
+		return -1;
+
 	return ploop_get_dev_by_delta(di->images[0]->file, NULL, NULL, out);
 }
 
@@ -1295,10 +1318,10 @@ static int ploop_mount_fs(struct ploop_mount_param *param)
 		return 0;
 
 	/* Two step mount
-	 * 1 mount ro and read balloon inode
+	 * 1 mount and find balloon inode
 	 * 2 remount with balloon_ino=ino
 	 */
-	if (mount(part_device, param->target, fstype, MS_RDONLY, param->mount_data)) {
+	if (mount(part_device, param->target, fstype, flags, param->mount_data)) {
 		ploop_err(errno, "Can't mount file system dev=%s target=%s data='%s'",
 				part_device, param->target, param->mount_data);
 		return SYSEXIT_MOUNT;
@@ -1719,7 +1742,7 @@ static int create_ploop_dev(int minor)
 	make_sysfs_dev_name(minor, device + 5, sizeof(device) - 5);
 	/* Create pair /dev/ploopN & /dev/ploopNp1 */
 	if (access(device, F_OK)) {
-		if (mknod(device, S_IFBLK, gnu_dev_makedev(PLOOP_DEV_MAJOR, minor))) {
+		if (mknod(device, S_IFBLK, makedev(PLOOP_DEV_MAJOR, minor))) {
 			ploop_err(errno, "mknod %s", device);
 			return SYSEXIT_MKNOD;
 		}
@@ -1730,7 +1753,7 @@ static int create_ploop_dev(int minor)
 	}
 	snprintf(devicep1, sizeof(devicep1), "%sp1", device);
 	if (access(devicep1, F_OK)) {
-		if (mknod(devicep1, S_IFBLK, gnu_dev_makedev(PLOOP_DEV_MAJOR, minor+1))) {
+		if (mknod(devicep1, S_IFBLK, makedev(PLOOP_DEV_MAJOR, minor+1))) {
 			ploop_err(errno, "mknod %s", devicep1);
 			return SYSEXIT_MKNOD;
 		}
@@ -2064,7 +2087,7 @@ err:
 	return ret;
 }
 
-int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *param, int flags)
+int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *param)
 {
 	int ret;
 	char **images;
@@ -2073,7 +2096,7 @@ int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *par
 	if (param->guid != NULL) {
 		if (find_image_by_guid(di, param->guid) == NULL) {
 			ploop_err(0, "Uuid %s not found", param->guid);
-			return SYSEXIT_PARAM;
+			return SYSEXIT_NOSNAP;
 		}
 		guid = param->guid;
 	} else
@@ -2100,6 +2123,47 @@ int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *par
 	return ret;
 }
 
+static int mount_fs(const char *dev, const char *target)
+{
+	char pdev[64];
+
+	if (get_partition_device_name(dev, pdev, sizeof(pdev)))
+		return SYSEXIT_MOUNT;
+
+	ploop_log(0, "Mounting %s at %s", pdev, target);
+	if (mount(pdev, target, DEFAULT_FSTYPE, 0, 0)) {
+		ploop_err(errno, "Can't mount dev=%s target=%s",
+				pdev, target);
+		return SYSEXIT_MOUNT;
+	}
+
+	return 0;
+}
+
+static int auto_mount_fs(struct ploop_disk_images_data *di, pid_t mntns_pid,
+		struct ploop_mount_param *param)
+{
+	int ret;
+	char target[PATH_MAX];
+
+	ret = get_temp_mountpoint(di->images[0]->file, 1, target, sizeof(target));
+	if (ret)
+		return ret;
+
+	param->target = strdup(target);
+
+	if (mntns_pid) {
+		ret = get_mntns_mount_dir(param->device, mntns_pid, NULL, 0);
+		if (ret < 0)
+			return SYSEXIT_SYS;
+
+		if (ret == 0) /* image mounted inside mnt namespace */
+			return mount_fs(param->device, target);
+	}
+
+	return ploop_mount_fs(param);
+}
+
 int auto_mount_image(struct ploop_disk_images_data *di,
 		struct ploop_mount_param *param)
 {
@@ -2111,7 +2175,7 @@ int auto_mount_image(struct ploop_disk_images_data *di,
 		return ret;
 	param->target = strdup(mnt);
 
-	return mount_image(di, param, 0);
+	return mount_image(di, param);
 }
 
 int ploop_mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *param)
@@ -2135,7 +2199,7 @@ int ploop_mount_image(struct ploop_disk_images_data *di, struct ploop_mount_para
 		goto err;
 	}
 
-	ret = mount_image(di, param, 0);
+	ret = mount_image(di, param);
 	if (ret == 0 && di->runtime->component_name == NULL)
 		merge_temporary_snapshots(di);
 err:
@@ -2179,6 +2243,7 @@ static int ploop_umount_fs(const char *mnt, struct ploop_disk_images_data *di)
 	 */
 	if (di != NULL && di->runtime->component_name == NULL)
 		store_statfs_info(mnt, di->images[0]->file);
+
 	ploop_log(0, "Unmounting file system at %s", mnt);
 	ret = do_umount(mnt);
 
@@ -2203,6 +2268,12 @@ int ploop_umount(const char *device, struct ploop_disk_images_data *di)
 
 	ret = ploop_stop_device(device);
 
+	if (di != NULL) {
+		get_temp_mountpoint(di->images[0]->file, 0, mnt, sizeof(mnt));
+		if (access(mnt, F_OK) == 0)
+			rmdir(mnt);
+	}
+
 	return ret;
 }
 
@@ -2225,7 +2296,7 @@ int ploop_umount_image(struct ploop_disk_images_data *di)
 		goto out;
 	}
 
-	ret = ploop_complete_running_operation(dev);
+	ret = complete_running_operation(di, dev);
 	if (ret) {
 		goto out;
 	}
@@ -2348,9 +2419,6 @@ int ploop_grow_device(const char *device, off_t new_size)
 		return SYSEXIT_PARAM;
 	}
 
-	if (check_size(new_size, blocksize, version))
-		return SYSEXIT_PARAM;
-
 	ploop_log(0, "Growing dev=%s size=%llu sectors (new size=%llu)",
 				device, (unsigned long long)size,
 				(unsigned long long)new_size);
@@ -2575,29 +2643,25 @@ int ploop_resize_image(struct ploop_disk_images_data *di, struct ploop_resize_pa
 			goto err;
 		mounted = 0;
 	} else {
-		ret = ploop_complete_running_operation(buf);
+		ret = complete_running_operation(di, buf);
 		if (ret)
 			goto err;
 
 		strncpy(mount_param.device, buf, sizeof(mount_param.device));
+
 		ret = get_mount_dir(mount_param.device, buf, sizeof(buf));
 		if (ret < 0) {
 			/* error message is printed by get_mount_dir() */
 			ret = SYSEXIT_SYS;
 			goto err;
 		} else if (ret > 0) { /* not mounted */
-			ret = get_temp_mountpoint(di->images[0]->file, 1, buf, sizeof(buf));
+			ret = auto_mount_fs(di, param->mntns_pid, &mount_param);
 			if (ret)
 				goto err;
-
-			mount_param.target = strdup(buf);
-			ret = ploop_mount_fs(&mount_param);
-			if (ret)
-				goto err;
-
 			umount_fs = 1;
 		} else
 			mount_param.target = strdup(buf);
+
 		mounted = 1;
 	}
 
@@ -3564,6 +3628,7 @@ int copy_delta(const char *src, const char *dst)
 		/* realloc */
 		cluster = S2B(vh->m_Sectors);
 		free(buf);
+		buf = NULL;
 		if (p_memalign(&buf, 4096, cluster)) {
 			ret = SYSEXIT_MALLOC;
 			goto out;
@@ -3606,8 +3671,7 @@ int copy_delta(const char *src, const char *dst)
 
 	ret = 0;
 out:
-	if (buf)
-		free(buf);
+	free(buf);
 	if (sfd >= 0)
 		close(sfd);
 	if (dfd >= 0)
