@@ -67,6 +67,10 @@
 #define ROUNDUP(size, blocksize) \
 	(((off_t)size + blocksize - 1) / blocksize * blocksize)
 
+#define BUILD_BUG_ON(condition) ((void)sizeof(char[1 - 2*!!(condition)]))
+
+#define PLOOP_LOCK_DIR	"/var/lock/ploop"
+
 /* PATH used by the library */
 #define DEF_PATH_ENV	"PATH=/sbin:/bin:/usr/sbin:/usr/bin:" \
 			"/usr/local/sbin:/usr/local/bin"
@@ -172,11 +176,6 @@ struct dump2fs_data {
 	uint32_t block_size;
 };
 
-enum {
-	PLOOP_MERGE_WITH_CHILD = 0,
-	PLOOP_MERGE_WITH_PARENT = 1,
-};
-
 /* flags for e2fsck() */
 enum e2fsck_flags {
 	E2FSCK_PREEN	= 1 << 0, /* -p */
@@ -199,6 +198,9 @@ static inline __u32 ploop_sec_to_ioff(off_t offSec, __u32 blocksize, int version
 	default:
 		assert(0);
 	}
+
+	// this unreachable code is here to satisfy broken compilers
+	return 0;
 }
 
 static inline off_t ploop_ioff_to_sec(__u32 iblk, __u32 blocksize, int version)
@@ -211,6 +213,9 @@ static inline off_t ploop_ioff_to_sec(__u32 iblk, __u32 blocksize, int version)
 	default:
 		assert(0);
 	}
+
+	// this unreachable code is here to satisfy broken compilers
+	return 0;
 }
 
 int gen_uuid_pair(char *uuid1, int len1, char *uuid2, int len2);
@@ -303,7 +308,7 @@ void __ploop_err(int err_no, const char *format, ...)
 	 })
 
 char *make_sysfs_dev_name(int minor, char *buf, int len);
-int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *param, int flags);
+int mount_image(struct ploop_disk_images_data *di, struct ploop_mount_param *param);
 PL_EXT int ploop_mount(struct ploop_disk_images_data *di, char **images,
 		struct ploop_mount_param *param, int raw);
 PL_EXT int replace_delta(const char *device, int level, const char *image);
@@ -409,7 +414,6 @@ int is_valid_blocksize(__u32 blocksize);
 int run_prg(char *const argv[]);
 #define HIDE_STDOUT	1 << 0	/* hide process' stdout */
 #define HIDE_STDERR	1 << 1	/* hide process' stderr */
-#define HIDE_NZ_EXIT	1 << 2	/* hide error about non-zero exit code */
 int run_prg_rc(char *const argv[], int hide_mask, int *rc);
 int p_memalign(void **memptr, size_t alignment, size_t size);
 PL_EXT int guidcmp(const char *p1, const char *p2);
@@ -428,7 +432,7 @@ int is_device_inuse(const char *dev);
 PL_EXT int get_delta_info(const char *device, struct merge_info *info);
 PL_EXT int merge_image(const char *device, int start_level, int end_level, int raw, int merge_top,
 		char **images, const char *new_delta);
-int ploop_merge_snapshot_by_guid(struct ploop_disk_images_data *di, const char *guid, int merge_mode, const char *new_delta);
+int ploop_merge_snapshot_by_guid(struct ploop_disk_images_data *di, const char *guid, const char *new_delta);
 int merge_temporary_snapshots(struct ploop_disk_images_data *di);
 
 PL_EXT int ploop_change_fmt_version(struct ploop_disk_images_data *di,
@@ -443,4 +447,7 @@ int do_replace_delta(int devfd, int level, int imgfd, __u32 blocksize,
 		const char *image);
 int copy_delta(const char *src, const char *dst);
 
+int complete_running_operation(struct ploop_disk_images_data *di,
+		const char *device);
+
 #endif
