@@ -129,7 +129,12 @@ static int do_delete_snapshot(struct ploop_disk_images_data *di, const char *gui
 			ploop_log(0, "ploop snapshot %s has been successfully deleted",
 				guid);
 	} else if (nelem == 1) {
-		ret = ploop_merge_snapshot_by_guid(di, guid, PLOOP_MERGE_WITH_CHILD, NULL);
+		const char *child_guid = ploop_find_child_by_guid(di, guid);
+		if (child_guid == NULL) {
+			ploop_err(0, "Can't find child of uuid %s", guid);
+			return SYSEXIT_PARAM;
+		}
+		ret = ploop_merge_snapshot_by_guid(di, child_guid, NULL);
 	} else if (!di->snapshots[snap_id]->temporary) {
 		ploop_log(1, "Warning: Unable to delete snapshot %s as there are %d references"
 				" to it; marking it as temporary instead",
@@ -180,10 +185,6 @@ int create_snapshot(const char *device, const char *delta, int syncfs)
 	__u32 blocksize;
 	int version;
 
-	ret = ploop_complete_running_operation(device);
-	if (ret)
-		return ret;
-
 	ret = get_image_param_online(device, &bdsize,
 			&blocksize, &version);
 	if (ret)
@@ -295,6 +296,9 @@ static int do_create_snapshot(struct ploop_disk_images_data *di,
 		return SYSEXIT_SYS;
 	else if (ret == 0) {
 		online = 1;
+		ret = complete_running_operation(di, dev);
+		if (ret)
+			return ret;
 	} else {
 		ret = get_image_param_offline(di, di->top_guid, &size, &blocksize, &version);
 		if (ret == SYSEXIT_OPEN && errno == EBUSY) {
@@ -450,7 +454,7 @@ int ploop_create_temporary_snapshot(struct ploop_disk_images_data *di,
 
 	mount_param.guid = param->guid;
 	mount_param.target = param->target;
-	ret = mount_image(di, &mount_param, 0);
+	ret = mount_image(di, &mount_param);
 	di->runtime->component_name = t;
 
 	if (ret)
@@ -473,7 +477,7 @@ err:
 	ploop_umount(mount_param.device, di);
 
 err_merge:
-	ploop_merge_snapshot_by_guid(di, param->guid, PLOOP_MERGE_WITH_CHILD, NULL);
+	ploop_merge_snapshot_by_guid(di, di->top_guid, NULL);
 
 err_unlock:
 	ploop_unlock_dd(di);
