@@ -412,8 +412,8 @@ static int pb_check_and_repair(int argc, char **argv, int repair)
 static void usage_discard(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon discard {-d DEVICE | -m MOUNT_POINT | DiskDescriptor.xml}\n"
-			"                     { [--automount] [--to-free SIZE] [--min-block MIN_SIZE] |\n"
-			"                       [--stat] }\n"
+			"                     { [--automount] [--to-free SIZE] [--min-block MIN_SIZE]\n"
+			"                       [--defrag] | [--stat] }\n"
 			"       DEVICE      := ploop device, e.g. /dev/ploop0\n"
 			"       MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"       SIZE        := NUMBER[KMGT] (maximum space to free)\n"
@@ -444,6 +444,7 @@ static int pb_discard(int argc, char **argv)
 		{ "min-block", required_argument, 0, 667 },
 		{ "stat", no_argument, 0, 668 },
 		{ "automount", no_argument, 0, 669 },
+		{ "defrag", no_argument, 0, 670 },
 		{},
 	};
 	struct ploop_disk_images_data *di = NULL;
@@ -477,6 +478,9 @@ static int pb_discard(int argc, char **argv)
 		case 669:
 			param.automount = 1;
 			break;
+		case 670:
+			param.defrag = 1;
+			break;
 		default:
 			usage_discard();
 			return SYSEXIT_PARAM;
