@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2008-2012, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2008-2013, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,9 +28,6 @@
 
 #define FMT "/sys/block/%s/pstat"
 
-static int clear;
-static int load;
-
 static void usage(void)
 {
 	fprintf(stderr, "Usage: ploop stat [-c | -l] -d DEVICE\n");
@@ -53,12 +50,15 @@ static DIR * open_sysfs_dir(char * devid)
 	return opendir(buf);
 }
 
-int main(int argc, char **argv)
+int plooptool_stat(int argc, char **argv)
 {
 	int i;
+	int clear = 0;
+	int load = 0;
 	DIR *dp;
 	struct dirent *de;
 	char * device = NULL;
+	int ret = 0;
 
 	while ((i = getopt(argc, argv, "cld:")) != EOF) {
 		switch (i) {
@@ -118,7 +118,7 @@ int main(int argc, char **argv)
 				perror("write");
 			close(fd);
 		}
-		return 0;
+		goto out;
 	}
 
 
@@ -133,7 +133,8 @@ int main(int argc, char **argv)
 		fd = open_sysfs_file(device, de->d_name, clear ? O_WRONLY : O_RDONLY);
 		if (fd < 0) {
 			perror("openat");
-			return SYSEXIT_SYSFS;
+			ret = SYSEXIT_SYSFS;
+			goto out;
 		}
 		if (clear) {
 			if (write(fd, "0\n", 2) <= 0)
@@ -142,12 +143,18 @@ int main(int argc, char **argv)
 			n = read(fd, buf, sizeof(buf)-1);
 			if (n < 0) {
 				perror("read");
-				return SYSEXIT_SYSFS;
+				close(fd);
+				ret = SYSEXIT_SYSFS;
+				goto out;
 			}
 			buf[n] = 0;
 			printf("%-20s\t%s", de->d_name, buf);
 		}
 		close(fd);
 	}
-	return 0;
+
+out:
+	closedir(dp);
+
+	return ret;
 }
